%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Function:        Rtro-rflecteur                                    %
%  Description:     Simulation d'un rtroreflecteur                     %
%  Dependicit:     refrac(), refrec(), polybox                         %
%  Auteur:          Stefan Bracher                                      %
%  Date:            31.10.2006                                          %
%  Statut:          complt                                             %
%                                                                       %
%  Syntaxe:         [a1, a2, b2, l]=retro(a0, b0, n0, n1, T, size)      %
%                                                                       %
%  Input:           a0: un point sur le faisceau incident               %
%                   b0: le vecteur du faisceau incident                 %
%                   n0: indexe de rfraction du milieu extrieure       %
%                   n1: indexe de rfraction du milieu de rtro         %
%                   T:  Matrice de transformation (position et          %
%                       orientation) du rtro*                          %
%                   size: dimension du rtro                            %
%                                                                       %
%                   *    x1  y1  z1  px                                 %
%                     T= x2  y2  z2  py                                 %
%                        x3  y3  z3  pz                                 %
%                        0   0   0   1                                  %
%                                                                       %
%  Output:          a1: le point d'entre du faisceau                   %   
%                   a2: le  point de sorti du faisceau                  %
%                   b2: le vecteur du faisceau sortant                  %
%                   l:  Chemin optique dans le rtro, si l=0, le        %   
%                       faisceau ne passait pas par le rtro            %
%                                                                       %
%   ATTENTION:     Si le faisceau frappe un coin du rtro, il est       %
%                  absorb, alors a2=[NaN NaN NaN]', b2=[NaN NaN NaN]'  %
%                                                                       %
%              Le systme du rtro:                                     %
%                                                                       %
%               Z                                                       %
%                          Y                                            %
%               X        X                                              %
%              XXX     XXX                                              %
%               X      X                                                %
%               X.    X                                                 %
%               X   .X                                                  %
%               X   X .                                                 %
%               X  X    .                                               %
%               X X       .                                             %
%               XX          .            X                              %
%               XXXXXXXXXXXXXXXXXXXXXXXXXXX X                           %
%                                        X                              %
%                                                                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [a1, a2, b2, l]=retro(a0, b0, n0, n1, T, size)

disp('Bonjour, je suis le rtro, voyons ce que je peux faire');
%---Definitions intrnes----------------------------------------------------
apex=[0 0 0]';                  % Le coin du rtro
x=[1 0 0]';                     % l'axe x (la normale de la face X)
y=[0 1 0]';                     % l'axe y (la normale de la face Y)
z=[0 0 1]';                     % l'axe z (la normale de la face Z)
ne=[1 1 1]'/norm([1 1 1]');     % la normale  la face E (la face entrant)
pe=[size 0 0]';                 % un point sur la face E
l=0;                            % Initialiser le chemin optique tant zro
inside=0;                       % Inside=0: le faisceau n'a pas entr le rtro
loop_count=0;                   % Compteur de boucle pour viter de boucles infinis
resolution=1e-12;               % Ce que est rgard tant zro (ou largeur des coins)
infinity=1e15;                  % Ce que est rgard tant infinit
cornerhit=0;                    % On n'as pas frapp un coin jusque ici
%-----------------------------------------------------------------------

%---Transformation de l'input dans le systme de coordonn rtro---------
p=trans_hom_point(a0, T);       % Transformer le point a0 au systme du rtro
n=trans_hom_vec(b0, T);         % Transformer le vecteur b0 au systme du rtro
%------------------------------------------------------------------------


%---Refraction initiale  la face E--------------------------------------
[a1, b1] = refrac(p, n, n0, n1, pe, ne); % Refraction initiale  la face E
%------------------------------------------------------------------------

%---Verification si le rtro a t touch et le faisceau est entr----------
dX=distance_point_plane(a1, x, apex);   % La distance entre le point d'entre et le plan X
dY=distance_point_plane(a1, y, apex);   % La distance entre le point d'entre et le plan Y
dZ=distance_point_plane(a1, z, apex);   % La distance entre le point d'entre et le plan Z

if ((dX>resolution)&&(dY>resolution)&&(dZ>0)&&(dot(a1-p, n)>resolution))
    % Le faisceau frappe le plan d'entre du rtro de l'exterieur
    disp('Ok, le faisceau frappe mon plan d entre de l exterieur -> Rfraction');
    inside=1;      % Le faisceau est entre
    p=a1;          % Un point sur le faisceau refract
    n=b1;          % La direction du faisceau refract
else
    % Le faisceu frappe le plan d'ntre de l'interieur
    % ou dehors des limites du rtro
    disp('Ooups, je suis trop petit ou le faisceau ne me touche pas au front, je peux rien faire...');
    l=NaN;              % tous ces valeurs n'existent pas
    p=[NaN NaN NaN]';
    n=[NaN NaN NaN]';
    a1=[NaN NaN NaN]';
    inside=0;      % Le faisceau n'est pas entre  
end
%------------------------------------------------------------------------


while (inside==1)
    % Pendant que le faisceau ce trouve dans le rtro
    loop_count=loop_count+1;        % Une msure de scurit pour eviter les boucles infinis
    too_close_warnings=0;           % Reinitialiser les alarmes d'tre trop proche
    
%---Refraction/Rflxion a tous les faces-----------------------------------
[ax, bx, pax, too_close]=mirror_reflexion(p , n, apex, x, resolution);     
                                                % Reflexion avec X, s'il y a lieu
too_close_warnings=too_close_warnings+too_close;% Mettre  jour le nombre d'alames
[ay, by, pay, too_close]=mirror_reflexion(p , n, apex, y, resolution);     
                                                % Reflexion avec Y, s'il y a lieu
too_close_warnings=too_close_warnings+too_close;% Mettre  jour le nombre d'alames
[az, bz, paz, too_close]=mirror_reflexion(p , n, apex, z, resolution);     
                                                % Reflexion avec Z, s'il y a lieu
too_close_warnings=too_close_warnings+too_close;% Mettre  jour le nombre d'alames
[ae, be, pae, too_close]=save_refraction(p, n, pe, -ne, n1, n0, resolution);
                                                % Refraction avec E, s'il y a lieu   
too_close_warnings=too_close_warnings+too_close;% Mettre  jour le nombre d'alames
%------------------------------------------------------------------------   
    
    
%---Dterminer la prmier face touch------------------------------------ 
cornerhit=1;    % Si long que aucune face  t touch on dois assumer qu'on frappe un coin
    %...Si X est touch en permier.......................................
     if ((pax<pay)&&(pax<paz)&&(pax<pae))
         disp('Ah, on est arriv  X, alors rflexion  X');
         l=l+pax;        % Mettre le chemin optique  jour
         p=ax;           % Le nouveau p
         n=bx;           % Le nouveau n
         cornerhit=0;    % Le premier face a t calucl -> no cornerhit
     end
    %....................................................................
    
    %...Si Y est touch en permier......................................
     if ((pay<pax)&&(pay<paz)&&(pay<pae))
         disp('Ah, on est arriv  Y, alors rflexion  Y');
         l=l+pay;        % Mettre le chemin optique  jour
         p=ay;                 % Le nouveau p
         n=by;                 % Le nouveau n
         cornerhit=0;          % Le premier face a t calucl -> no cornerhit
     end
    %...................................................................
    
    %...Si Z est touch en permier......................................
     if ((paz<pax)&&(paz<pay)&&(paz<pae))
         disp('Ah, on est arriv  Z, alors rflexion  Z');
         l=l+paz;         % Mettre le chemin optique  jour
         p=az;                  % Le nouveau p
         n=bz;                  % Le nouveau n
         cornerhit=0;           % Le premier face a t calucl -> no cornerhit
     end
    %...................................................................
    
    %...Si E est touch en permier......................................
     if ((pae<pax)&&(pae<pay)&&(pae<paz))
         disp('Ah, on est arriv  E, alors rfraction  E');
         l=l+pae;         % Mettre le chemin optique  jour
         p=ae;                  % Le nouveau p
         n=be;                  % Le nouveau n
         inside=0;              % On quitte le rtro
         cornerhit=0;           % Le premier face a t calucl -> no cornerhit
     end
    %...................................................................
%-----------------------------------------------------------------------

%---Coin-test----------------------------------------------------------- 
if (too_close_warnings>1)
    % Plus que un alarme d'approche a sonn, donc on se trouve sur un coin
    % (Si on est TRES proche du plus qu'une face, on est sur un coin)
    cornerhit=1;
end

if (cornerhit==1)
    % Un coin a t frapp
    disp('Ooops, le faisceau a t absorb par un de mes coins, dsol!');
    p=[NaN NaN NaN]';
    n=[NaN NaN NaN]';
    l=NaN;
    inside=0;
    end
%-----------------------------------------------------------------------   

%---Boucle-test----------------------------------------------------------
    if (loop_count>4)  
        % Plus que la nombre maximale des incidences avec surfaces (=4) est
        % arriv
     inside=0;
     disp('Ooops: Un erreur innatendu est arriv!');
    end    
%-----------------------------------------------------------------------   

%---Error debugging code------------------------------------------------
% p
% n
% pax 
% pay 
% paz 
% pae
% pae
%-----------------------------------------------------------------------

end

%---Rflexion totele test-------------------------------------------------
% Il se peut que,  cause d'une dimension limit, le faisceau frappe la
% face E, mais, en principe devrait encore une fois etre reflechi  une
% autre face. Dans la pluspart de ces cas, l'ange d'incident sur E va etre
% trs petit, ce que cause une rflction totale, qui se manifeste dans un
% rsultat immaginaire

if ~isreal(n)
    disp('Ooops, une rflexion totale est arriv le faisceau ne rentre pas, dsol!');
    p=[NaN NaN NaN]';
    n=[NaN NaN NaN]';
    l=NaN;
end
%-----------------------------------------------------------------------

disp('Le faisceau n est plus  l interieur de moi, j essaye de vous donner les informations demand...A la prochaine!');

%---Transformation de l'output dans le systme de coordonn globale---------
a1=trans_hom_point(a1, inv(T));       % Le point d'entre
a2=trans_hom_point(p, inv(T));       % Le point de sortie
b2=trans_hom_vec(n, inv(T));           % Le vecteur du faisceau sortant
%--------------------------------------------------------------------------